/**
* @file		AHString.h
* @brief	NX
*
* @author	ISHINO k16
* @date		2003-07-01 XFDefs.h Ɨ
*
* $Id: AHString.h 3427 2019-09-10 14:18:23Z k16 $
*
* Copyright (C) 2002-2009, Antenna House, Inc.  All Rights Reserved.
*/

#ifndef AHSTRING_H__
#define AHSTRING_H__

#include "AHBase.h"
//#ifdef CCCTRL_ENABLE_HASH_SETMAP
#ifdef AH_HAVE_UNORDERED_SETMAP
#include <unordered_map>
#elif defined AH_HAVE_HASH_SETMAP
#include <hash_map>
#endif

#if (U_ICU_VERSION_MAJOR_NUM > 4 || (U_ICU_VERSION_MAJOR_NUM == 4 && U_ICU_VERSION_MINOR_NUM >= 2))
#define AHSTRING_HAVE_STRINGPREP
#include <unicode/usprep.h>
#include <vector>
#endif


////////////////////////////////////////////////////////////////////////////////
// ICU Cu UnicodeString NXgp

#include <unicode/unistr.h>
#include <unicode/uversion.h>

namespace AHCommonNS {

/**
 * NX.
 * ICU  UnicodeString  AHString ̖OŎgp.
 */
typedef UnicodeString AHString;

/**
 * 萔߂̃}N
 *
 * @param s	p[^ ASCII Ȃ C ł邱.
 */
#define AHSTR(s)	UNICODE_STRING(s, sizeof(s) - 1)
/*			p^ UChar32 */
#define AHSTRC(c)	AHString(UChar32(c))

/**
 * 󕶎\V{
 */
class AHCOMMON_API AHEmptyString : public AHString
{
public:
	static const AHEmptyString EMPTY_VALUE;
	AHEmptyString() : AHString() {}
	static const AHString& getEmptyString() { return EMPTY_VALUE; }
};
//#define AH_EMPTY_STRING  ((const AHCommonNS::AHString&)AHCommonNS::AHEmptyString::EMPTY_VALUE)
#define AH_EMPTY_STRING		AHCommonNS::AHEmptyString::getEmptyString()
//#define AH_EMPTY_STRING  AHString()

/**
 * Null\V{.
 *
 * ICU UnicodeString ɂ Bogus l Null Ƃėp.
 * isBogus() \bhɂ, eXg邱Ƃł. isEmpty()  true ƂȂ.
 * , rZq ==  AH_EMPTY_STRING Ɣr false ɂȂ.
 *
 * AHString((UChar *)NULL, 0, 0) ł Bogus ɂȂȂ (ICU2.6)
 * Length==0 łȂ isEmpty()==true ɂȂȂ.
 * AHString((UChar *)~0, 0, -1) Ȃ Bogus ɂ Empty ɂȂ邯ǁAhNXS.
 */
class AHCOMMON_API AHBogusString : public AHString
{
public:
	static const AHBogusString BOGUS_VALUE;
	AHBogusString() : AHString((UChar *)NULL, 0, 0) { setToBogus(); }
	static const AHString& getBugusString() { return BOGUS_VALUE; }
};
//#define XF_NULL_STRING		XFBogusString()
//#define XF_BOGUS_STRING		XFBogusString()	// 
//#define AH_NULL_STRING		((const AHCommonNS::AHString&)AHCommonNS::AHBogusString::BOGUS_VALUE)
#define AH_NULL_STRING		AHCommonNS::AHBogusString::getBugusString()
#define AH_BOGUS_STRING		AH_NULL_STRING

/**
 * 2 ̕A + Zq.
 *
 * ICU2.8 unistr.h Ɏς
 */
#if !(U_ICU_VERSION_MAJOR_NUM > 2 || U_ICU_VERSION_MINOR_NUM >= 8)
inline AHString operator+(const AHString& lhs, const AHString& rhs) {
	AHString str = lhs;
	return str += rhs;
}
#endif


//#ifdef CCCTRL_ENABLE_HASH_SETMAP
#ifdef AH_HAVE_UNORDERED_SETMAP

struct AHString_hash {

	size_t operator()(const AHString& str) const { return str.hashCode(); }
};

#elif defined AH_HAVE_HASH_SETMAP	// AHCommonȊOŉpł悤
/**
 * hash_map p
 */
class AHString_less : public std::less<AHString> {
public:
	bool operator()(const AHString& str1, const AHString& str2) const {
		return str1 < str2;
	}
};
class AHString_compare : public stdext::hash_compare<AHString, AHString_less> {
public:
	size_t operator()(const AHString& str) const {
		return str.hashCode();
	}
	bool operator()(const AHString& str1, const AHString& str2) const {
#if defined AH_WIN_MSC_ && _MSC_VER >= 1700
		return _Getcomp()(str1, str2);
#else
		return comp(str1, str2);
#endif
	}
};
#endif

#ifdef AHSTRING_HAVE_STRINGPREP
// s output = BogusString
inline void ahStringPrep(UStringPrepProfileType type,const AHString& input,AHString& output,bool allowUnassigned)  {
	if (input.isEmpty()) {
		output = input;
		return;
	}
	UErrorCode status = U_ZERO_ERROR;
	UStringPrepProfile * profile = usprep_openByType(type, &status);
	if (U_FAILURE(status)) {
		output.setToBogus();
		return;
	}
	std::vector<UChar> outBuf(input.length()*2,0); // Ȃ鎖͖Ô2{̒ƂĂB
	status = U_ZERO_ERROR;
	UParseError parseError;
	parseError.line = 0;
	parseError.offset = 0;
	int32_t size = usprep_prepare(profile, input.getBuffer(), input.length(), &outBuf[0], (int32_t)outBuf.size(),
								allowUnassigned ? USPREP_ALLOW_UNASSIGNED: USPREP_DEFAULT
								,&parseError,&status);
	usprep_close(profile);
	if (U_FAILURE(status) || parseError.offset>0 || size > (int)outBuf.size()) {
		output.setToBogus();
		return;
	}

	if (size <= 0) {
		output = AHString();
		return;
	}
	output = AHString(&outBuf[0], size);
}
#endif

} // namespace AHCommonNS

#endif // AHSTRING_H__
